import os, sys, math, random
import pygame
from pygame.locals import *
from math import *

LogicalFPS = 64.0
TicksPerFrame = (1000.0 / LogicalFPS)
MaxFrameSkip = 2

class Particle:
    def __init__ (self, pos, color = (250, 25, 25)):
        self.Start(pos, color)
        
    def Start(self, pos, color):
        self.color = color
        
        self.pos = (0.0, 0.0)
        self.pos = pos
        self.direction = random.randint(0,359)
        self.age = random.randint(1, 6)
        self.size = 9 - (self.age)
        
        self.velocity = (0.0, 0.0)
        self.velocity = (sin(self.direction) * self.age * 40, cos(self.direction) * self.age * 40)
        
        self.age = self.age / 4.0
        
        self.colorage = (self.color[0] / self.age, self.color[1] / self.age, self.color[2] / self.age)
        
    def Compute(self, dt):
        self.pos = (self.pos[0] + (self.velocity[0] * dt), self.pos[1] + (self.velocity[1] * dt))
        if (640 < self.pos[0] < 0):
            self.age = 0
        elif (480 < self.pos[1] < 0):
            self.ago = 0
        self.age -= dt
        
    def Draw(self, screen):
        color = (self.color[0] - (self.colorage[0] * self.age), self.color[1] - (self.colorage[1] * self.age), self.color[2] - (self.colorage[2] * self.age))
        pygame.draw.circle(screen, color, (int(self.pos[0]), int(self.pos[1])), int(self.size), 0)
        
        

class SimObj:
    def __init__(self, pos, mass, vel, type):
        self.pos = (0.00, 0.00)
        self.pos = (float(pos[0]), float(pos[1]))
        self.mass = mass #5 #kg
        self.accel = (0.00, 0.00)
        self.velocity = (0.0, 0.0)
        self.velocity = vel
        self.rotation = 0.00
        self.Stationary = False
        self.IsFragment = False
        self.IsRocket = False
        self.IsParticle = False
        self.IsPlayer = False
        self.AddParticle = False
        self.width = 32
        self.type = type
        self.frame = 0
        self.framecount = 0.00
        self.emitcount = 0.00
        self.checked = False
        
        if self.type == 1:
            self.width = 128
        elif self.type == 2:
            self.width = 64
        elif self.type == 3:
            self.width = 64
        elif self.type == 4:
            self.width = 32
        elif self.type == 5:
            self.width = 32
        elif self.type == 6:
            self.width = 32
        elif self.type == 7:
            self.width = 32
        elif self.type == 0:
            self.width = 22
        
        #types:
        # 0 - nothing or fragment
        # 1 - big earth
        # 2 - lil earth
        # 3 - fire planet
        # 4- girly planet
        # 5 - death star
        # 6 - heart planet
        # 7 - DO NOT HIT planet
        
    def UpdatedPos(self, dt):
        tempvel = (self.velocity[0] + (self.accel[0] * dt), self.velocity[1] + (self.accel[1] * dt))
        return (self.pos[0] + (self.velocity[0] * dt), self.pos[1] + (self.velocity[1] * dt))
    
    def Compute(self, dt):
        #apply accel
        self.velocity = (self.velocity[0] + (self.accel[0] * dt), self.velocity[1] + (self.accel[1] * dt))
        
        #bounce off of walls plz
        if (self.pos[0] + (self.velocity[0] * dt) > (640 - (self.width * 0.5))) or (self.pos[0] + (self.velocity[0] * dt) < (self.width * 0.5)):
           self.velocity = (-self.velocity[0], self.velocity[1])
            
        if (self.pos[1] + (self.velocity[1] * dt) > (480 - (self.width * 0.5))) or (self.pos[1] + (self.velocity[1] * dt) < (self.width * 0.5)):
           self.velocity = (self.velocity[0], -self.velocity[1])
        
        self.pos = (self.pos[0] + (self.velocity[0] * dt), self.pos[1] + (self.velocity[1] * dt))
        
        if self.IsRocket == True:
            self.framecount += dt
            self.emitcount += dt
            
            if self.emitcount > 0.1:
                self.emitcount = 0.0
                self.AddParticle = True
                
            if self.framecount > 0.05:
                self.framecount = 0
                self.frame += 1
                if self.frame > 7:
                    self.frame = 0
        
        #just in case?
        if self.pos[0] > (640 - (self.width / 2)):
            self.pos = (640 - (self.width / 2), self.pos[1])
        if self.pos[0] < (self.width / 2):
            self.pos = ((self.width / 2), self.pos[1])
        if self.pos[1] > (480 - (self.width / 2)):
            self.pos = (self.pos[0], 480 - (self.width / 2))
        if self.pos[1] < (self.width / 2):
            self.pos = (self.pos[0], (self.width / 2))
        
        # print "vel: "
        # print self.velocity
        # print "accel: "
        # print self.accel
        
class Game:
    def __init__(self):
        #create planets during level start
        self.simobjs = []
        self.particles = []
        self.LaunchMissile = False
        self.cursorpos = (0,0)
        self.playerRot = 0.0
        self.playerpos = (0.00,0.00)
        self.playervel = (0.00,0.00)
        self.poweringShot = False
        self.shotPower = 0.0
        pygame.mouse.set_visible(0)
        self.level = 1
        self.score = 0
        self.maxscore = 1
        self.congratscountdown = 0.00
        
        #load data
        self.tiles = pygame.image.load(os.path.join('data', 'planet-tiles.png')).convert_alpha()
        self.bg = pygame.image.load(os.path.join('data', 'stars-bg.png')).convert_alpha()
        self.bigfont = pygame.font.Font(os.path.join('data', 'Vera.ttf'), 26)
        self.thefont = pygame.font.Font(os.path.join('data', 'Vera.ttf'), 12)
        self.bigexplode = pygame.mixer.Sound(os.path.join('data', 'explode2.wav'))
        self.lilexplode = pygame.mixer.Sound(os.path.join('data', 'explode1.wav'))
        self.launch = pygame.mixer.Sound(os.path.join('data', 'launch.wav'))
        self.beep = pygame.mixer.Sound(os.path.join('data', 'beep.wav'))
    
    def ClampDegrees(self, degrees):
        if degrees > 360:
            degrees -= 360.0
        elif degrees < 0:
            degrees += 360.0
            
        return degrees
    
    def FigureThisOut(self):
        rot = (math.atan2(-(self.cursorpos[1]-self.playerpos[1]),-(self.cursorpos[0]-self.playerpos[0])))
        
        #result = (sin(rot), cos(rot))
        result = (-cos(rot), -sin(rot))
        
        #if (result[0] > 0 and result[1] > 0) or (result[0] < 0 and result[1] < 0):
        #    result = (-(result[0]), -(result[1]))
        
        #result = (1 - result[0], 1 - result[1])
        
        return result
    
    def DoInputStuff(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                sys.exit()
            elif event.type == KEYDOWN:
                if (event.key == K_ESCAPE):
                    return False
                    
            elif (event.type == MOUSEBUTTONDOWN):
                if self.poweringShot == False and self.score < self.maxscore:
                    self.poweringShot = True
                
                #self.ButtonDown()
            elif (event.type == MOUSEBUTTONUP):
                if self.poweringShot == True and self.score < self.maxscore:
                    self.poweringShot = False
                    self.launch.play()
                    
                    
                    #rot = self.ClampDegrees(rot)
                    result = self.FigureThisOut()
                      
                    startpos = (self.playerpos[0] + (result[0] * 50), self.playerpos[1] + (result[1] * 50))
                    power = self.shotPower * 15
                    
                    startvel = ((result[0] * power), (result[1] * power))
                    
                    self.shotPower = 0.0
                    
                    test = SimObj(startpos, random.randint(100, 250), startvel, 0)
                    test.IsRocket = True
                    test.width = 18
                    self.simobjs.append(test)
                elif self.poweringShot == True:
                    self.poweringShot = False
                    
            elif (event.type == MOUSEMOTION):
                self.cursorpos = event.pos
                #self.playerRot = degrees(atan2(self.cursorpos[0], self.cursorpos[1]))
                
                #self.playerRot = 
                
        return True
    
    def SetRotation(self, pos):
        for object in self.simobjs:
            if object.IsPlayer == 1:
                self.playerRot = float(degrees(math.atan2((object.pos[1]-pos[1]),-(object.pos[0]-pos[0])))) - 90.0
                self.playerRot = self.ClampDegrees(self.playerRot)
    
    def ComputeForces(self):
        # f_g = G * (m1 + m2) / r^2 
        # r is distance between the midpoints -- losing the ^2 might be fun
        #  G = 6.67 x 10^-11  -- or whatever works
        for object in self.simobjs:
            object.accel = (0.0,0.0)
            for otherobj in self.simobjs:
                if otherobj != object:
                    dist = self.FindDistance(object.pos, otherobj.pos)
                    if (dist > (object.width + otherobj.width) / 2.0):
                        accelC = (80 * (object.mass + otherobj.mass)) / (dist)
                        accelC = -accelC / object.mass
                        a = object.pos[0] - otherobj.pos[0]
                        b = object.pos[1] - otherobj.pos[1]
                        c = sqrt( (a*a) + (b*b) )
                        angleX = acos(a/c)
                        angleY = asin(b/c)
                        object.accel = (object.accel[0] + (cos(angleX) * accelC), object.accel[1] + (sin(angleY) * accelC))
                        object.rotation = angleY
                        
    def ApplyForces(self, dt):
    
        self.SetRotation(self.cursorpos)
    
        for object in self.simobjs:
            object.Compute(dt)
            if object.AddParticle == True:
                object.AddParticle = False
                self.particles.append(Particle((object.pos[0] - (object.velocity[0] * 0.2), object.pos[1] - (object.velocity[1] * 0.2)), ((40,40,40))))
            
        for part in self.particles:
            part.Compute(dt)
            if part.age <= 0:
                self.particles.remove(part)
            
    def GetMagnitude(self, vel):
        return sqrt(vel[0] * vel[0] + vel[1] * vel[1])
        
    def GetVector(self, input):
        mag = self.GetMagnitude(input)
        if mag == 0:
            mag = 1
        return (input[0] / mag, input[1] / mag)
        
    def DotProduct(self, one, other):
        return one[0] * other[0] + one[1] * other[1]
        
    def MultByScalar(self, thescalar, input):
        return (thescalar * input[0], thescalar * input[1])
        
    def CollideSphere2(self, b1, b2):
        # Compute unit normal and unit tangent vectors
        v_n = (b2.pos[0] - b1.pos[0], b2.pos[1] - b1.pos[1]) # v_n = normal vec. - a vector normal to the collision surface
        v_un = self.GetVector(v_n) # unit normal vector
        v_ut = (-v_un[1], v_un[0]) #unit tangent vector
        
        # Compute scalar projections of velocities onto v_un and v_ut
        v1n = self.DotProduct(v_un, b1.velocity) # Dot product
        v1t = self.DotProduct(v_ut, b1.velocity)
        v2n = self.DotProduct(v_un, b2.velocity)
        v2t = self.DotProduct(v_ut, b2.velocity)
        
        # Compute new tangential velocities
        v1tPrime = v1t # Note: in reality, the tangential velocities do not change after the collision
        v2tPrime = v2t
        
        # Compute new normal velocities using one-dimensional elastic collision equations in the normal direction
        # Division by zero avoided. See early return above.
        v1nPrime = (v1n * (b1.mass - b2.mass) + 2. * b2.mass * v2n) / (b1.mass + b2.mass)
        v2nPrime = (v2n * (b2.mass - b1.mass) + 2. * b1.mass * v1n) / (b1.mass + b2.mass)
        
        # Compute new normal and tangential velocity vectors
        v_v1nPrime = self.MultByScalar(v1nPrime, v_un) # Multiplication by a scalar
        v_v1tPrime = self.MultByScalar(v1tPrime, v_ut)
        v_v2nPrime = self.MultByScalar(v2nPrime, v_un)
        v_v2tPrime = self.MultByScalar(v2tPrime, v_ut)
        
        # Set new velocities in x and y coordinates
        b1.velocity = (v_v1nPrime[0] + v_v1tPrime[0], v_v1nPrime[1] + v_v1tPrime[1])
        b2.velocity = (v_v2nPrime[0] + v_v2tPrime[0], v_v2nPrime[1] + v_v2tPrime[1])
        # b1.setVX(v_v1nPrime.x() + v_v1tPrime.x());
        # b1.setVY(v_v1nPrime.y() + v_v1tPrime.y());
        # b2.setVX(v_v2nPrime.x() + v_v2tPrime.x());
        # b2.setVY(v_v2nPrime.y() + v_v2tPrime.y());
            
    def CollideSphere(self, obj1, obj2):
        #angle = atan ((ballY (k) - ballY (i)) / ((ballX (k) - ballX (i)))) % angle of hit
        stuff = 0
        stuff = ((obj2.pos[0] - obj1.pos[0]))
        if (stuff == 0):
            stuff = 1
        angle = atan ((obj2.pos[1]) - obj1.pos[1]) / stuff
        
        cosAngle = cos (angle)
        sinAngle = sin (angle)

        #speedAfterHit = (2 * ((ballXSpeed (i) * cosAngle + ballYSpeed (i) * sinAngle) - (ballXSpeed (k) * cosAngle + ballYSpeed (k) * sinAngle))) / (ballMass (i) + ballMass (k))
        speedAfterHit = (2*((obj1.velocity[0] * cosAngle + obj1.velocity[1] * sinAngle) - (obj2.velocity[0] * cosAngle + obj2.velocity[1] * sinAngle))) / (obj1.mass + obj2.mass)
        
        obj1.velocity = (obj1.velocity[0] - (speedAfterHit * obj2.mass * cosAngle), obj1.velocity[1] - (speedAfterHit * obj2.mass * sinAngle))
        obj2.velocity = (obj2.velocity[0] + (speedAfterHit * obj1.mass * cosAngle), obj2.velocity[1] + (speedAfterHit * obj1.mass * sinAngle))
        # ballXSpeed (i) -= speedAfterHit * ballMass (k) * cosAngle
        # ballYSpeed (i) -= speedAfterHit * ballMass (k) * sinAngle
        # ballXSpeed (k) += speedAfterHit * ballMass (i) * cosAngle
        # ballYSpeed (k) += speedAfterHit * ballMass (i) * sinAngle 
    
    def Fragment(self, obj1, obj2, pieces):
        for x in range(pieces):
            origangle = (atan2(obj1.velocity[0], obj1.velocity[1]))
            #object.accel[0] + (cos(angleX) * accelC), object.accel[1] + (sin(angleY) * accelC)
            
            origangle -= 15
            origangle += ((30) / pieces) * ( x + 1 )
            
            #origangle = self.ClampDegrees(origangle)
            
            newx = (sin(origangle) * 100)
            newy = (cos(origangle) * 100)
            newposx = obj2.pos[0] + (sin(origangle) * 50)
            newposy = obj2.pos[1] + (cos(origangle) * 50)
            
            #randx = (random.choice(range(70, 130)) * 0.01) * object.velocity[0]
            #randy = (random.choice(range(70, 130)) * 0.01) * object.velocity[1]
            
            newobj = SimObj((newposx, newposy), 5, (newx,newy), random.randint(0,5))
            newobj.IsFragment = True
            newobj.width = 25
            
            self.simobjs.append(newobj)
    
    def Asplode(self, pos, rangenum):
        for x in range(rangenum):
            self.particles.append(Particle(pos))
    
    def CollisionStuff(self, dt):
    
        for object in self.simobjs:
            object.checked = False
            if object.IsPlayer == True:
                self.playerpos = object.pos
                self.playervel = object.velocity
    
        for object in self.simobjs:
            if object.checked == False:
                for otherobj in self.simobjs:
                    if otherobj != object and (otherobj.checked == False):
                        dist = self.FindDistance(object.UpdatedPos(dt), otherobj.UpdatedPos(dt))
                        sizetogether = (object.width + otherobj.width) * 0.5
                        if dist <= (sizetogether):
                            #collided!
                            #print 'kerbams'
                            
                            if object.IsFragment and otherobj.IsFragment == False and otherobj.type > 1:
                                self.lilexplode.play()
                                
                                self.score += 1
                                self.Asplode(otherobj.pos, 50)
                                self.simobjs.remove(object)
                                self.simobjs.remove(otherobj)
                                
                            elif otherobj.IsFragment and object.IsFragment == False and object.type > 1:
                                self.lilexplode.play()
                                
                                self.score += 1
                                self.Asplode(object.pos, 50)
                                self.simobjs.remove(object)
                                self.simobjs.remove(otherobj)
                            
                            elif otherobj.IsRocket and object.IsRocket:
                                self.bigexplode.play()
                                self.Asplode(object.pos, 100)
                                self.simobjs.remove(object)
                                self.simobjs.remove(otherobj)
                            
                            elif object.IsRocket and otherobj.IsFragment:
                                self.lilexplode.play()
                                self.Asplode(otherobj.pos, 10)
                                self.simobjs.remove(object)
                                self.simobjs.remove(otherobj)
                                
                            elif otherobj.IsRocket and object.IsFragment:
                                self.lilexplode.play()
                                self.Asplode(object.pos, 10)
                                self.simobjs.remove(object)
                                self.simobjs.remove(otherobj)
                                
                            elif object.IsRocket and (otherobj.type != 0):
                                #print 'rocket punched'
                                self.bigexplode.play()
                                self.Fragment(object, otherobj, random.randint(2,4))
                                self.Asplode(otherobj.pos, 80)
                                
                                self.simobjs.remove(object)
                                self.simobjs.remove(otherobj)
                                    
                            elif otherobj.IsRocket and object.type != 0:
                                #print 'punched by rocket'
                                self.bigexplode.play()
                                self.Fragment(otherobj, object, random.randint(2,4))
                                self.Asplode(object.pos, 80)
                                
                                self.simobjs.remove(object)
                                self.simobjs.remove(otherobj)
                            
                            else:
                                #bouncy
                                #otherobj.velocity = (-otherobj.velocity[0], -otherobj.velocity[1])
                                #object.velocity = (-object.velocity[0], -object.velocity[1])
                                self.CollideSphere2(object, otherobj)
                                
                                otherobj.checked = True
                                object.checked = True
                            
                            
                            
    def RenderObjs(self, screen):
    
        screen.lock()
    
        for part in self.particles:
            part.Draw(screen)
        
        screen.unlock()        
        
        for object in self.simobjs:
            #print object.pos
            #pygame.draw.circle(screen, (255,0,0), object.pos, object.mass / 10, 0)
            if (object.IsRocket == True):
                rotation = degrees(atan2(-object.velocity[0], -object.velocity[1]))
                temp1 = pygame.Surface((32,64), self.tiles.get_flags(), self.tiles.get_bitsize())
                temp1 = temp1.convert_alpha(temp1)
                temp1.blit(self.tiles, (0,0), (192 + (object.frame * 32),0,32,64))
                temp2 = pygame.transform.rotate(temp1, rotation)
                newpos = (object.pos[0] - (temp2.get_rect().center[0]) , object.pos[1] - (temp2.get_rect().center[1]))
                #temp2.set_colorkey((0,0,0), RLEACCEL)
                #screen.blit(temp2, (object.pos[0] + (therect[0] * 0.5), object.pos[1] + (therect[1] * 0.5)))
                screen.blit(temp2, newpos)
                
                #pygame.draw.circle(screen, (255,0,0), object.pos, object.width)
                #print object.pos[0] - (temp2.get_rect()[0])
                #print object.pos[1] - (temp2.get_rect()[1])
            elif (object.IsFragment == True):
            
                therect = (0,0,128,128)
                if object.type == 0:
                    therect = (256, 64, 32, 32)
                if object.type == 1:
                    therect = (256, 96, 32, 32)
                if object.type == 2:
                    therect = (288, 64, 32, 32)
                if object.type == 3:
                    therect = (288, 96, 32, 32)
                if object.type == 4:
                    therect = (320, 64, 32, 32)
                if object.type == 5:
                    therect = (320, 96, 32, 32)
                if object.type == 6:
                    therect = (352, 64, 32, 32)
                if object.type == 7:
                    therect = (352, 96, 32, 32)
                if object.type == 8:
                    therect = (384, 64, 32, 32)
                if object.type == 9:
                    therect = (384, 96, 32, 32)
                if object.type == 10:
                    therect = (416, 64, 32, 32)
                if object.type == 11:
                    therect = (416, 96, 32, 32)

                screen.blit(self.tiles, (object.pos[0] - 16, object.pos[1] - 16), therect)    
            elif object.type == 0:
                #is player
                #rotation = degrees(atan2(-object.velocity[0], -object.velocity[1]))
                temp1 = pygame.Surface((40,40), self.tiles.get_flags(), self.tiles.get_bitsize())
                temp1 = temp1.convert_alpha(temp1)
                temp1.blit(self.tiles, (0,0), (448,40,40,40))
                temp2 = pygame.transform.rotate(temp1, self.playerRot)
                newpos = (object.pos[0] - (temp2.get_rect().center[0]) , object.pos[1] - (temp2.get_rect().center[1]))
                #temp2.set_colorkey((0,0,0), RLEACCEL)
                #screen.blit(temp2, (object.pos[0] + (therect[0] * 0.5), object.pos[1] + (therect[1] * 0.5)))
                screen.blit(temp2, newpos)
                screen.blit(self.tiles, (object.pos[0] - 20, object.pos[1] - 20), (448,0,40,40))
                
            else:
            #is a planet
                therect = (0,0,128,128)
                if object.type == 1:
                    therect = (0,0,128,128)
                elif object.type == 2:
                    therect = (128,64,64,64)
                elif object.type == 3:
                    therect = (128,0,64,64)
                elif object.type == 4:
                    therect = (192,64,32,32)
                elif object.type == 5:
                    therect = (192,96,32,32)
                elif object.type == 6:
                    therect = (224,64,32,32)
                    
                screen.blit(self.tiles, (object.pos[0] - (object.width * 0.5), object.pos[1] - (object.width * 0.5)), therect)    
                
        #types:
        # 0 - not a planet
        # 1 - big earth
        # 2 - lil earth
        # 3 - fire planet
        # 4- girly planet
        # 5 - death star
        # 6 - heart planet
                
                #pygame.draw.circle(screen, (255,0,0), object.pos, object.mass / 10, 0)
    
    def drawStatus(self, screen, statustext):
        # statusBar = pygame.Surface((640, 16)).convert()
        # statusBar.fill((20,20,20))
        # screen.blit(statusBar, (0, 0))
    
        text = self.thefont.render(statustext, 1, (255,255,255))
        textpos = text.get_rect(centerx = screen.get_width()/2, centery = ((text.get_height()/2)))
        screen.blit(text, textpos)
    
    def RenderGUI(self, screen):
    
        # result = self.FigureThisOut()
        # pos = (self.playerpos[0] + result[0] * 50, self.playerpos[1] + result[1] * 50)
        
        # pygame.draw.circle(screen, (0,0,255), pos, 5, 0)
    
        self.drawStatus(screen, "level: %d       points: %d       points needed: %d" % (self.level, self.score, self.maxscore))
    
        if (632 > self.cursorpos[0] > 8) and (472 > self.cursorpos[1] > 8):
            screen.blit(self.tiles, (self.cursorpos[0] - 8, self.cursorpos[1] - 8), (488,0,16,16))
            
        if self.poweringShot:
            pygame.draw.rect(screen, (0,200,0), (10, 20, (self.shotPower * 75) + 10, 30))
            
        if self.congratscountdown > 0:
            
            ElapsedTicks = 0.00
            LastTicks = pygame.time.get_ticks()
        
            while self.congratscountdown > 0:
                self.thetime = pygame.time.get_ticks()
                ElapsedTicks = self.thetime - LastTicks
                LastTicks = self.thetime
                
                screen.fill((0,0,0))
                text = self.bigfont.render("LEVEL %d" % (self.level), 1, (255,255,255))
                textpos = text.get_rect(centerx = screen.get_width()/2, centery = ((screen.get_height()/2)))
                screen.blit(text, textpos)
                
                text = self.thefont.render("in %f" % (self.congratscountdown / 1000.0), 1, (255,255,255))
                textpos = text.get_rect(centerx = screen.get_width()/2, centery = (((screen.get_height()+100)/2)))
                screen.blit(text, textpos)
                
                for event in pygame.event.get():
                    if event.type == pygame.QUIT: 
                        sys.exit()
                    elif event.type == KEYDOWN:
                        self.congratscountdown = 0
                    elif event.type == MOUSEBUTTONUP:
                        self.congratscountdown = 0
                    elif (event.type == MOUSEMOTION):
                        self.cursorpos = event.pos
                
                if (632 > self.cursorpos[0] > 8) and (472 > self.cursorpos[1] > 8):
                    screen.blit(self.tiles, (self.cursorpos[0] - 8, self.cursorpos[1] - 8), (488,0,16,16))
                
                pygame.display.flip()
                # delay time
                pygame.time.wait(25)
                self.congratscountdown -= (ElapsedTicks)
            
            self.score = 0
            self.InitLevel(self.level)
            self.AddPlayer(self.playerpos, self.playervel)
    
    def AddPlayer(self, pos, vel):
        player = SimObj(pos, 200, vel, 0)
        player.IsPlayer = 1
        player.width = 22
        self.simobjs.append(player)
    
    def InitLevel(self, level):
    
        self.score = 0
        
        if level == 1:
            self.maxscore = 1
            self.simobjs.append(SimObj((300,300), 2000, (0.0,0.0), 1))
            self.simobjs.append(SimObj((100,100), 600, (45.0, 0.0), 4))
            self.simobjs.append(SimObj((150,150), 600, (12.0, -20.0), 5))
            self.playerpos = ((400,400))
            self.playervel = ((20.0,10.0))
        
        elif level == 2:
            self.maxscore = 2
            self.simobjs.append(SimObj((200,370), 2000, (0.0,0.0), 2))
            self.simobjs.append(SimObj((100,500), 1700, (45.0, 0.0), 3))
            self.simobjs.append(SimObj((450,150), 600, (12.0, -20.0), 6))
            self.playerpos = ((100,300))
            self.playervel = ((50.0,10.0))
            
        elif level == 3:
            self.maxscore = 2
            self.simobjs.append(SimObj((300,300), 2000, (2.0,0.0), 1))
            self.simobjs.append(SimObj((100,400), 3000, (0.0,6.0), 1))
            self.simobjs.append(SimObj((500,100), 2000, (7.0,0.0), 1))
            self.simobjs.append(SimObj((380,250), 600, (12.0, 20.0), 5))
            self.simobjs.append(SimObj((100,100), 600, (45.0, 0.0), 4))
            self.simobjs.append(SimObj((50,500), 1700, (45.0, 0.0), 6))
            self.playerpos = ((100,300))
            self.playervel = ((50.0,10.0))
            
        elif level == 4:
            self.maxscore = 5
            self.simobjs.append(SimObj((300,300), 2500, (2.0,0.0), 1))
            self.simobjs.append(SimObj((100,400), 1000, (0.0,6.0), 5))
            self.simobjs.append(SimObj((500,100), 1300, (7.0,0.0), 3))
            self.simobjs.append(SimObj((450,150), 600, (12.0, 20.0), 6))
            self.simobjs.append(SimObj((100,100), 600, (45.0, 0.0), 4))
            self.simobjs.append(SimObj((500,300), 690, (2.0,0.0), 6))
            self.simobjs.append(SimObj((200,400), 599, (0.0,6.0), 5))
            self.simobjs.append(SimObj((560,200), 690, (7.0,0.0), 4))
            self.playerpos = ((200,300))
            self.playervel = ((-20.0,-10.0))
            
        elif level == 5:
            self.maxscore = 2
            
            self.simobjs.append(SimObj((100,100), 3000, (45.0, 0.0), 1))
            self.simobjs.append(SimObj((500,300), 690, (2.0,0.0), 4))
            self.simobjs.append(SimObj((200,400), 599, (0.0,6.0), 5))
            self.simobjs.append(SimObj((560,200), 690, (7.0,0.0), 4))
            self.playerpos = ((400,200))
            self.playervel = ((0.0,10.0))
            
        elif level == 6:
            self.maxscore = 3
            
            self.simobjs.append(SimObj((400,400), 100, (50.0, 0.0), 2))
            self.simobjs.append(SimObj((50,50), 1000, (50.0, 0.0), 3))
            self.simobjs.append(SimObj((590,50), 1000, (0.0,-50.0), 3))
            self.simobjs.append(SimObj((50,430), 1000, (0.0,50.0), 3))
            self.simobjs.append(SimObj((590,430), 1000, (-50.0,0.0), 3))
            self.playerpos = ((320,240))
            self.playervel = ((0.0,0.0))
            
        elif level == 7:
            self.maxscore = 5
            
            self.simobjs.append(SimObj((320,250), 10000, (50.0, 0.0), 1))
            self.simobjs.append(SimObj((590,200), 3000, (0.0,20.0), 4))
            self.simobjs.append(SimObj((250,430), 3000, (0.0,10.0), 5))
            self.simobjs.append(SimObj((590,430), 3000, (-20.0,0.0), 6))
            self.simobjs.append(SimObj((390,50), 3000, (0.0,-10.0), 4))
            self.simobjs.append(SimObj((50,130), 3000, (0.0, 5.0), 2))
            self.simobjs.append(SimObj((590,50), 3000, (20.0,0.0), 6))
            self.simobjs.append(SimObj((50,430), 3000, (0.0,10.0), 5))
            self.playerpos = ((100,100))
            self.playervel = ((20.0,20.0))
            
        elif level == 8:
            self.maxscore = 3
            
            self.simobjs.append(SimObj((100,100), 10000, (25.0, 0.0), 1))
            self.simobjs.append(SimObj((500,300), 2000, (2.0,0.0), 4))
            self.simobjs.append(SimObj((200,400), 2000, (0.0,6.0), 5))
            self.simobjs.append(SimObj((560,200), 2000, (7.0,0.0), 4))
            self.simobjs.append(SimObj((560,400), 2000, (7.0,0.0), 2))
            self.playerpos = ((400,200))
            self.playervel = ((0.0,10.0))
            
        elif level == 9:
            self.maxscore = 1
            self.simobjs.append(SimObj((300,300), 2000, (50.0, 20.0), 1))
            self.simobjs.append(SimObj((150,150), 600, (-50.0, -20.0), 2))
            self.playerpos = ((400,400))
            self.playervel = ((20.0,10.0))
        
            
        
    def FindDistance(self, pos1, pos2):
        return sqrt(((pos2[0]-pos1[0])*(pos2[0]-pos1[0])) + ((pos2[1]-pos1[1])*(pos2[1]-pos1[1])))
    
    def CheckForWin(self, screen):
        if self.score >= self.maxscore:
            for obj in self.simobjs:
                if obj.IsRocket:
                    self.simobjs.remove(obj)
                    
            if len(self.particles) == 0:
            
                self.simobjs = []
                self.particles = []
                self.level += 1
                #self.showcongrats = True
                self.congratscountdown = 2500
            
                if self.level == 10:
                    return 1
        return 0
    
    def CheckForLose(self):
        planetcount = 0
        
        for object in self.simobjs:
            if object.IsRocket == False and object.IsFragment == False and object.type > 1:
                planetcount += 1
        
        if planetcount == 0 and self.score < self.maxscore:
            for obj in self.simobjs:
                if obj.IsRocket:
                    self.simobjs.remove(obj)
                    
            if len(self.particles) == 0:
                return 1
        
        return 0
    
    def Go(self, screen):
        #init
        
        ElapsedTicks = 0.00
        LastTicks = pygame.time.get_ticks()
        
        self.InitLevel(self.level)
        self.AddPlayer(self.playerpos, self.playervel)
        
        while (1):
            self.thetime = pygame.time.get_ticks()
            ElapsedTicks += self.thetime - LastTicks
            LastTicks = self.thetime
            
            if (ElapsedTicks > (TicksPerFrame * 4)):
                ElapsedTicks = 0

            frames = 0
            
            if (ElapsedTicks > TicksPerFrame):
                while ((ElapsedTicks > TicksPerFrame) and (frames < MaxFrameSkip)):
                    ElapsedTicks -= TicksPerFrame
                    timestep = 1.0 / LogicalFPS
                    
                    #do stuff
                    if (self.DoInputStuff() == False):
                        return 0
                    
                    #sim stuff (woo)
                    self.ComputeForces()
                    self.CollisionStuff(timestep)
                    
                    self.ApplyForces(timestep)
                    
                    if self.poweringShot:
                        if self.shotPower < 8:
                            self.shotPower += (timestep * 5)
                    
                    frames += 1
                    
            if self.CheckForWin(screen):
                return 1;
                
            if self.CheckForLose():
                return -1;
            
            #rendar
            #screen.fill((0,0,0))
            screen.blit(self.bg, (0,0))
            
            self.RenderObjs(screen)
            self.RenderGUI(screen)
            
            pygame.display.flip()
            # delay time
            pygame.time.wait(10)
            
            #return
            
            
            